//##############################################################################################
// Status eiens Senec Speichers anzeigen
// (C) Ralf Amandi, Avarix GmbH
//  26.10.2020 V1.0
//  04.02.2021 V1.1 Bug Fixes und Sonnenaufgang
//  07.07.2021 V1.21 Exception ausgabe verbessert
//##############################################################################################
var colGaugeBar = "#1F7C94"; //31,124,148 dunkelblau
var colGaugeFrame = "#176173"; // 23,97,115
var colGaugeInner = "#E7F5FA"; // 231,245,250 Hellblau
var colGaugeText = "#101010"; 

var colLedGreen = ["#004020","#00592D","#00E070"]; //rahmen, aus, an
var colLedYel = ["#4A1404","#6B1D05","#F88C6D"];
var colLedYel = ["#434203","#555304","#FAF898"]; // 250,248,152
var colLedRed = ["#4D0207","#64020A","#FB4050"];

	

function formatInt(val,stellen) 
{
	var s = val.toFixed(0);
	if (s.length<stellen) s="00000".substr(5+s.length-stellen)+s;	
	return s;
}

function formatFloat(val,nk)
{
	var v = Math.round( val * Math.pow(10,nk));
	var s = Math.abs(v).toString(); // ohne Vorzeichen
	var l = s.length;
	if (l<nk+1) {
		s="00000".substr(4+l-nk)+s; // nullen davor (max 5)
		l = s.length;
	}
	var p=l-nk; 
	if (nk>0) s=s.substr(0,p)+","+s.substr(p);
	while (p>3) {
		p=p-3;
		s=s.substr(0,p)+"."+s.substr(p);
	}
	if (v<0) s="-"+s;
	return s;
}

function fmt2(v)
{
	if (v<10) return "0"+v; else return v;
}
function formatDate(d)
{
	return fmt2(d.getDate())+"."+fmt2(d.getMonth()+1)+"."+d.getFullYear()+" "+fmt2(d.getHours())+":"+fmt2(d.getMinutes())+":"+fmt2(d.getSeconds());
}

function formatTimestamp(d)
{
	return d.getFullYear()+"-"+fmt2(d.getMonth()+1)+"-"+fmt2(d.getDate())+" "+fmt2(d.getHours())+":"+fmt2(d.getMinutes())+":"+fmt2(d.getSeconds());
}

function decodeFormatted(senVal,nk)
{
	return formatFloat(decodeVal(senVal),nk);
}

function setFormatted(id, senVal,nk, unit)
{
	var tag = document.getElementById(id);
	if (tag) {
		tag.innerText = formatFloat(decodeVal(senVal),nk)+" "+unit;
	}
}

function setText(id, text)
{
	var tag = document.getElementById(id);
	if (tag) {
		if (text && text !="" && text.substr(0,3)=="st_") text=text.substr(3);
		tag.innerText = text;
	}
}


function toBinary(val)
{
	var s = "";
	var b = 1;
	for(var j=0;j<32;j++) {
		//if (j==4 || j==8 ||j==12||j==16||j==20||j==24||j==28) s=s+" ";
		if (j==8 ||j==16||j==24) s=s+" ";
		if ((val & b)!=0) {
			s=s+"1";
		} else {
			s=s+"0";
		}
		b=b*2;
	}
	return s;
}
		
//############ GAUGE ##################
function drawGauge(id,min, act,max)
{
	var p=(act-min)/(max-min); // percent
	var cv = document.getElementById(id);
	var displ = cv.getContext("2d");
	displ.fillStyle = colGaugeInner; 
	displ.fillRect(0,0,cv.width,cv.height);
	displ.strokeStyle = colGaugeFrame; 
	displ.strokeRect(0,0,cv.width,cv.height);
	gmax=(cv.width-2)
	gw=gmax*p;
	if (gw>gmax) gw=gmax;
	displ.fillStyle = colGaugeBar; 
	displ.fillRect(1,1,gw,cv.height-2);
	displ.fillStyle = colGaugeText; 
	displ.fillText(formatFloat(p*100,1)+" %",gmax/2-10,cv.height/2+4);
	
}

//########################################################
//############ Diagramme #################################
//########################################################

function createChart(idCanvas, title, unitX, unitY, minX,minY,maxX,maxY)
{
	var canvas = document.getElementById(idCanvas);
	var ch = {
		cv  :  canvas,
		ctx  : canvas.getContext("2d"),
		title : title,
		min  : { x : minX, y : minY },
		max  : { x : maxX, y : maxY },
		np	 : { x:0,y:0},	// Nullpunkt in pixel
		xf :0, yf :0, xo : 0, yo : 0, // Umrechung Wert -> Pixel px=x*xf+xo
		//
		smoothing : 5,  // 5% der breite
		smoothRange : 0,
		recalcRange : 0,
		
		// Werte
		lines : [],
		// Beschriftung der Achen, scale: Umrechung in Teilstriche 
		axisX : { scale: 1, strichWert: 1, nk : 0, unit : unitX, txtOfsX : -3, txtOfsY : 10, from:0, to:1 },
		axisY : { scale: 1, strichWert: 1, nk : 0, unit : unitY, txtOfsX : 4, txtOfsY : 4, from:0, to:1  },
		
		addLine : function (title,color,scaleX,scaleY) {
			var l = {
				chart : this,
				text : title,
				col : color,
				scale : { x: (scaleX)?scaleX:1, y: (scaleY)?scaleY:1 },
				pix : [],
				cntAddVal : 0, // nach Recalc
				
				addValue : function ( vx,vy) {
					if (vy == null || vy==NaN || vy==Infinity)  return;
					if (vx == null || vx==NaN || vx==Infinity)  return;
					this.cntAddVal++;
					vx = vx / this.scale.x;
					vy = vy / this.scale.y;
					var vdx =(this.pix.length==0)?0:(vx-this.pix[this.pix.length-1].x);
					if (this.pix.length==1) this.pix[0].dx =vdx;
					
					this.pix.push( {x:vx,y:vy,dx:vdx} );
					with (this.chart) {
						if (vx<min.x) { min.x = vx; }
						if (vx>max.x) { max.x = vx; }
						if (vy<min.y) { min.y = vy; }
						if (vy>max.y) { max.y = vy; }					
					}
				},
				
				recalcValues : function (xrange) {
					var nPix =[];
					var vx=0, vy=0, vdx=0, vs=0;
					for(var i=0; i<this.pix.length; i++) {
						with (this.pix[i]) {
							var s= Math.floor(x / xrange); 
							if (s != vs) { // Abschnittswechsel
								if (vdx>0) nPix.push( {x:vx,y:vy/vdx,dx:vdx} );
								vx=0; vy=0; vdx=0; vs=s;
							}
							vdx=vdx+dx;
							vy = vy+y*dx;
							vx = x;
						}
					}
					if (vdx>0) nPix.push( {x:vx,y:vy/vdx,dx:vdx} );					
					var a = JSON.stringify(this.pix);
					var n = JSON.stringify(nPix);
					//console.log(this.text+"-"+xrange);
					//console.log(a);
					//console.log(n);
					this.pix= nPix;
					this.cntAddVal =0;
				},
				ltrim : function (xval) {					
					while (this.pix.length>0 && this.pix[0].x <= xval) this.pix.shift();
					if (this.pix.length>0 &&  this.pix[0].x<this.chart.min.x) { this.chart.min.x = this.pix[0].x; }
				}
			}
			this.lines.push(l);
			return l;
		},
		
		calcUnits : function (axis, min,max) {
				var sw = (max-min) / axis.scale;
				var v=Math.floor(Math.log10( sw ));
				if (v<-2) { v=-2; }
				axis.strichWert=Math.pow(10,v);
				
				// 0.2, 0.5 oder 1 lassen
				if ( v > -2 ) {
					if ( sw / axis.strichWert <2) {
						axis.strichWert = axis.strichWert / 5; 
						v=v-1;
					} else if ( sw / axis.strichWert <5) {
						axis.strichWert = axis.strichWert / 2; 
						v=v-1;
					} 
				}
		
				// Skalierung anpassen?
				if (axis.unit == "s" && sw >= 120) {
					axis.unit="m"; 
					axis.scale=axis.scale*60;
					axis.strichWert = 1;
				} else if (axis.unit == "m" && sw >= 120) {
					axis.unit="h"; 
					axis.scale=axis.scale*60;
					axis.strichWert = 1;
				}
				if ( sw>1000 ) {
					if (axis.unit.substr(0,1)=='K') {
						axis.unit = "M" + axis.unit.substr(1);
					} else if (axis.unit.substr(0,1)=='M') {
						axis.unit= "G" + axis.unit.substr(1);
					} else {
						axis.unit= "K" + axis.unit;
					}	
					axis.scale=axis.scale*1000;
					axis.strichWert = axis.strichWert / 1000;
					v=v-3;					
				}				
				
				// Berechnen
				axis.nk=(v<0)?-v:0;
				axis.from = Math.floor(min / axis.scale / axis.strichWert) * axis.strichWert;
				axis.to   = Math.ceil (max / axis.scale / axis.strichWert) * axis.strichWert; 
		}
		,
		calcCoordinateSystem : function() {
			with (this) {
				calcUnits(axisX,min.x, max.x);
				calcUnits(axisY,min.y, max.y);
				
				var xrange=(axisX.to - axisX.from) * axisX.scale;
				var yrange=(axisY.to - axisY.from) * axisY.scale;
				var aa = 5; var ae = 10; // abstand der Teilungsstriche vom Pfeilanfang und ende
				if (axisX.from>0 && axisX.to>0) { // gleicher quadrant
					np.x = 10;
					xf = (cv.width-aa-ae-np.x) / xrange; // Faktor
					xo = np.x+aa-axisX.from* axisX.scale*xf;
					//xo = np.x+aa-min.x*xf
				} else { // x = x*xf +xo
					if (axisX.from == 0) { aa=aa+5; }
					xf = (cv.width-aa-ae) / xrange; // 
					xo = aa-min.x*xf;
					np.x = Math.round(xo);
				}
				// oben ist 0 und unten cv.height, daher yf negativ
				aa=10; ae =10;
				if (axisY.from>0 && axisY.to>0) { // gleicher quadrant
					np.y = cv.height-10;
					yf = - (np.y-aa-ae) / yrange; // Faktor
					yo = (np.y-aa)-min.y*yf;
				} else { // x = x*xf +xo
					yf = - (cv.height-aa-ae) / yrange; 
					yo = (cv.height-aa)-min.y*yf;
					np.y = Math.round(yo);
				}
			}
		},
		
		formatLabel : function (val,axis)
		{
			if (axis.unit == "h") {
				var h =Math.floor(val);
				var m = Math.floor( (val-h)*60);
				return  s = formatInt(h,2)+":"+formatInt(m,2);
			} else {
				return formatFloat(val,axis.nk);
			}
		},
		
		drawCoordinateSystem : function () {
			with (this) {
				var p2 = cv.width-1;
				ctx.fillStyle = "#000000";  // Textfarbe
				ctx.beginPath();
				ctx.fillText(title,40,10);
				// X Pfeil
				ctx.moveTo(0,np.y);
				ctx.lineTo(p2,np.y);
				ctx.moveTo(p2-3,np.y-3);
				ctx.lineTo(p2,np.y);
				ctx.lineTo(p2-3,np.y+3);
				ctx.fillText(axisX.unit, cv.width-15,np.y -5);
				// Y Pfeil
				ctx.moveTo(np.x,cv.height-1);
				ctx.lineTo(np.x,0);
				ctx.moveTo(np.x-3,3);
				ctx.lineTo(np.x,0);
				ctx.lineTo(np.x+3,3);
				ctx.fillText(axisY.unit, np.x+axisY.txtOfsX, 10);
				// Striche
				for(var w = axisX.from; w <=axisX.to; w=w+axisX.strichWert) {
					p2 = w*axisX.scale*xf+xo;
					if (Math.round(p2) != np.x) {
						ctx.moveTo(p2,np.y-2);
						ctx.lineTo(p2,np.y+2);
						ctx.fillText(formatLabel(w,axisX),p2+axisX.txtOfsX,np.y+axisX.txtOfsY);
					}					
				}
				for(var w = axisY.from; w <=axisY.to; w=w+axisY.strichWert) {
					p2 = w*axisY.scale*yf+yo;
					if (Math.round(p2) != np.y) {
						ctx.moveTo(np.x-2,p2);
						ctx.lineTo(np.x+2,p2);
						ctx.fillText(formatLabel(w,axisY),np.x+axisY.txtOfsX,p2+axisY.txtOfsY);
					}
				}
				ctx.strokeStyle = "#000000";
				ctx.stroke();
					
			}
		},
		
		drawLine : function(line,y, smoothRange) {
			with (this) {
				if (line.pix.length<2) return;
				ctx.beginPath();
				ctx.strokeStyle = line.col;
				ctx.moveTo(line.pix[0].x*xf+xo,line.pix[0].y*yf+yo);
				var lx, ly, ldx, minx;
				for (var i=1; i< line.pix.length;i++) {
					lx = line.pix[i].x;
					ly = line.pix[i].y;
					if (smoothRange>0) { // MIttelwerte
						ldx = line.pix[i].dx;
						ly = ly * ldx;
						minx = lx-smoothRange;
						j=i-1;
						while (j>=0 && line.pix[j].x>=minx) {
							ly = ly + line.pix[j].y * line.pix[j].dx;
							ldx = ldx + line.pix[j].dx;
							j=j-1;
						}
						ly = ly / ldx;
					}
					
					lx = lx*xf+xo; ly = ly*yf+yo;
					ctx.lineTo(lx,ly);
				}
				ctx.stroke();
				ctx.fillStyle = line.col; 
				ctx.fillText(line.text,ctx.canvas.width-40,y);
			}			
		},
		
		draw : function () {
			this.ctx.fillStyle = "#FFFFFF";
			this.ctx.fillRect(0,0,this.cv.width,this.cv.height);
			this.calcCoordinateSystem();
			this.drawCoordinateSystem();
			this.smoothRange = (this.max.x-this.min.x) * this.smoothing / 100;
			for(var i=0;i<this.lines.length;i++) {
				this.drawLine(this.lines[i],8+i*8,this.smoothRange);
			}
			
		},
		
		checkRecalc : function (maxValues) {
			if (! maxValues) maxValues = this.cv.width;
			var needRecalc = false;
			var xrange = (this.max.x-this.min.x) / maxValues;
			if (xrange > 0 ) {
				if (this.recalcRange == 0 || xrange> this.recalcRange*5) { // Erst ändern, wenn es sich lohnt
					this.recalcRange =xrange;
					needRecalc = true;
				}
				// alle lines
				for(var i=0;i<this.lines.length;i++) {
					with (this.lines[i]) {
						if (cntAddVal>maxValues || needRecalc) recalcValues ( this.recalcRange);
					}
				}			
			}
		},
		
		// alle Werte kleiner xval aus Liste entfernen!
		ltrim : function (xval) {					
			// alle lines
			this.min.x = this.max.x; // min hochsetzen.
			for(var i=0;i<this.lines.length;i++) {
				this.lines[i].ltrim(xval);				
			}			
		}		
		
	};
	return ch
}
//############ SONNEN AUFGANG/UNTERGANG ##################
// Quelle: https://www.herber.de/forum/archiv/776to780/778752_Sonnenaufgang.html
function Stunde2Str(Stunde, Datum)
{
	var h=Math.floor(Stunde);
	Datum.setUTCHours(h,0,Math.floor((Stunde-h)*3600));
	return fmt2(Datum.getHours())+":"+fmt2(Datum.getMinutes());
}

function Sonnenaufgang ( Laengengrad, Breitengrad, Datum)
{
	// Umrechen 0..360 => 0..2PI (Bogenmaß)
	var BreitengradBog = Breitengrad * Math.PI / 180;
	
	//Sonnenaufgang bei -50 Bogenminuten
	var H_aufgang = (-50 / 60) * Math.PI / 180;
	
	//Tag des Jahres
	var JahrAnf = new Date(Datum.getFullYear(),0,0); // 00.01.YYYY
	var Jahrestag = Math.floor((Datum.getTime() - JahrAnf.getTime())/1000/3600/24);
	
	//Breitengrad, über dem die Sonne Mittags senkrecht steht
	var Deklination = 0.40954 * Math.sin(0.0172 * (Jahrestag - 79.35));
	
	//Differenz zum Mittag in Stunden berechnen
	var dummy = (Math.sin(H_aufgang) - Math.sin(BreitengradBog) * Math.sin(Deklination)) / (Math.cos(BreitengradBog) * Math.cos(Deklination));
	var DiffMittag = 12 * (Math.atan( -dummy / Math.sqrt(1 -dummy * dummy)) + 2 * Math.atan(1)) / Math.PI;
	var DiffWozMoz = -0.1752 * Math.sin(0.03343 * Jahrestag + 0.5474) - 0.134 * Math.sin(0.018234 * Jahrestag - 0.1939);
	return (12 - DiffMittag - DiffWozMoz -  Laengengrad * 24 / 360) // Stunde 0..24; 360 => 24h
}


function Sonnenuntergang ( Laengengrad, Breitengrad, Datum)
{
	// Umrechen 0..360 => 0..2PI (Bogenmaß)
	var BreitengradBog = Breitengrad * Math.PI / 180;
	
	//Sonnenaufgang bei -50 Bogenminuten
	var H_aufgang = (-50 / 60) * Math.PI / 180;
	
	//Tag des Jahres
	var JahrAnf = new Date(Datum.getFullYear(),0,0); // 00.01.YYYY
	var Jahrestag = Math.floor((Datum.getTime() - JahrAnf.getTime())/1000/3600/24);
	
	//Breitengrad, über dem die Sonne Mittags senkrecht steht
	var Deklination = 0.40954 * Math.sin(0.0172 * (Jahrestag - 79.35));
	
	//Differenz zum Mittag in Stunden berechnen
	var dummy = (Math.sin(H_aufgang) - Math.sin(BreitengradBog) * Math.sin(Deklination))  / (Math.cos(BreitengradBog) * Math.cos(Deklination));
	var DiffMittag = 12 * (Math.atan( -dummy / Math.sqrt(1 -dummy * dummy)) + 2 * Math.atan(1)) / Math.PI;
	var DiffWozMoz = -0.1752 * Math.sin(0.03343 * Jahrestag + 0.5474) - 0.134 * Math.sin(0.018234 * Jahrestag - 0.1939);
	return (12 + DiffMittag - DiffWozMoz -  Laengengrad * 24 / 360) // Stunde 0..24; 360
}


